#!/usr/local/bin/tcsh

# version 1.1 sept 2005
#

if ( $# != 4 ) then
echo "Usage: leon.tcsh input_file query_sequence_name output_aln_file output_log_file"
exit
endif

setenv LEONPATH /home/julie/LEON

# convert input alignment to suitable format for external programs
$LEONPATH/convseq $1 $$.msf gcg 
$LEONPATH/convseq $1 $$.rsf rsf 
$LEONPATH/convseq $1 $$.tfa fasta 

# run cluspack (secator) to get sequence clusters
/biolo/secator/cluspack $$.msf -data_type=alignment -clustering_method=bionj -nb_clusters=secator -filtering_distance=0.01

# run ncoils to predict coiled coils
setenv COILSDIR $LEONPATH
$LEONPATH/ncoils-osf -c < $$.tfa >& $$.coils

# predict transmembrane helices, low complexity regions
$LEONPATH/resbias $$.tfa -v > $$.bias
$LEONPATH/bias2relacs $$.tfa $$.bias $$.clu $$.int1.rsf
$LEONPATH/coils2relacs $$.int1.rsf $$.coils $$.clu $$.int2.rsf

# calculate core blocks with rascal algorithm
$LEONPATH/comp_query $$.tfa $$.clu $2 > $$.cmp
$LEONPATH/query_seqerrs $$.tfa $$.clu > $$.seqerrs
$LEONPATH/query2relacs $$.int2.rsf $$.cmp $$.seqerrs $2 $$.int3.rsf

# chain core blocks into conserved regions
# set threshold parameters x=5, d=40, T=280, L=21
$LEONPATH/chain_blocks $$.int3.rsf $2 $$.int4.rsf 5 40 280 21 >$4

#output the sequences in either gcg or fasta format
$LEONPATH/convseq $$.int4.rsf $3 gcg
#convseq $$.int4.rsf $3 fasta
\rm $$*
