#!/usr/local/bin/tcsh

if ( $# != 4 ) then
echo "Usage: leon.tcsh input_file query_sequence_name output_aln_file output_log_file"
exit
endif

# convert input alignment to suitable format for external programs
/home/julie/leon/convseq $1 $$.msf gcg 
/home/julie/leon/convseq $1 $$.rsf rsf 
/home/julie/leon/convseq $1 $$.tfa fasta 

# run cluspack (secator) to get sequence clusters
/home/julie/leon/cluspack $$.msf -data_type=alignment -clustering_method=bionj -nb_clusters=secator -filtering_distance=0.01

# run ncoils to predict coiled coils
setenv COILSDIR .
/home/julie/leon/ncoils-osf -c < $$.tfa >& $$.coils

# predict transmembrane helices, low complexity regions
/home/julie/leon/resbias $$.tfa -v > $$.bias
/home/julie/leon/bias2relacs $$.tfa $$.bias $$.clu $$.int1.rsf
/home/julie/leon/coils2relacs $$.int1.rsf $$.coils $$.clu $$.int2.rsf

# calculate core blocks with rascal algorithm
/home/julie/leon/comp_query $$.tfa $$.clu $2 > $$.cmp
/home/julie/leon/query_seqerrs $$.tfa $$.clu > $$.seqerrs
/home/julie/leon/query2relacs $$.int2.rsf $$.cmp $$.seqerrs $2 $$.int3.rsf

# chain core blocks into conserved regions
# set threshold parameters x=5, d=40, T=280, L=21
/home/julie/leon/chain_blocks $$.int3.rsf $2 $$.int4.rsf 5 40 280 21 >$4

#output the sequences in either gcg or fasta format
/home/julie/leon/convseq $$.int4.rsf $3 gcg
#convseq $$.int4.rsf $3 fasta
\rm $$*
